
#include "chat_line.hpp"
#include <iostream>


chat_line::chat_line() : _params()
{
}


string chat_line::get_command()
{
	if (_params.count("command") == 0)
		return "";
	return _params["command"];
}


void chat_line::set_command(string command)
{
	_params["command"] = command;
}


string chat_line::get_username()
{
	if (_params.count("username") == 0)
		return "";
	return _params["username"];	
}


void chat_line::set_username(string username)
{
	_params["username"] = username;
}


string chat_line::get_password()
{
	if (_params.count("password") == 0)
		return "";
	return _params["password"];	
}


void chat_line::set_password(string password)
{
	_params["password"] = password;
}


void chat_line::set_to(string to)
{
	_params["to"] = to;
}


string chat_line::get_to()
{
	if (_params.count("to") == 0)
		return "";
	return _params["to"];
}


void chat_line::set_from(string from)
{
	_params["from"] = from;
}


string chat_line::get_from()
{
	if (_params.count("from") == 0)
		return "";
	return _params["from"];
}


void chat_line::set_chat_line(string line)
{
	_params["chat"] = line;
}


string chat_line::get_chat_line()
{
	if (_params.count("chat") == 0)
		return "";
	return _params["chat"];		
}


bool chat_line::empty()
{
	return _params.empty();
}


string chat_line::to_string()
{
	string result;
	for (map<string, string>::const_iterator it = _params.begin(); it != _params.end(); it++)
		result += it->first + "=" + it->second + ",";
	return result;
}


bool chat_line::parse(string request)
{
	size_t begin = 0;
	while (begin <= request.size() - 1)
	{
		size_t end = request.find(chat_line::PARAM_SEP, begin);
		if (end == string::npos)
			break;
			
		string name_value = request.substr(begin, end - begin);
		size_t pos = name_value.find(chat_line::NAME_VALUE_SEP);
		if (pos == string::npos)
			return false;
		string name = trim(name_value.substr(0, pos));
		string value = trim(name_value.substr(pos + 1));
		_params[name] = value;
		
		begin = end + 1;
	}
	
	string name_value = request.substr(begin);
	size_t pos = name_value.find(chat_line::NAME_VALUE_SEP);
	if (pos == string::npos)
		return false;
	string name = trim(name_value.substr(0, pos));
	string value = trim(name_value.substr(pos + 1));
	_params[name] = value;
	
	return true;
}


string chat_line::format()
{
	string str;
	for (map<string, string>::const_iterator it = _params.begin(); it != _params.end(); it++)
	{
		if (str == "")
			str = it->first + chat_line::NAME_VALUE_SEP + it->second;
		else
			str += chat_line::PARAM_SEP + it->first + chat_line::NAME_VALUE_SEP + it->second;
	}
	return str;
}


string chat_line::trim(string str)
{
	size_t begin = str.find_first_not_of(WHITESPACES);
	size_t end = str.find_last_not_of(WHITESPACES);
	return str.substr(begin, end - begin + 1);
}


const string chat_line::PARAM_SEP = "&";

const string chat_line::NAME_VALUE_SEP = "=";

const string chat_line::WHITESPACES = " \t\n\r";
