
#include "handler.hpp"
#include "chat_line.hpp"
#include <iostream>
#include <string>
#include <boost/bind.hpp>
#include <boost/thread.hpp>


using std::cout;
using std::endl;
using std::string;
using boost::shared_ptr;


map<string, string> handler::_members;

mutex handler::_mutex_users;

map<long, string> handler::_users;

mutex handler::_mutex_inbox;

map<string, vector<handler::From> > handler::_inbox;


handler::handler(shared_ptr<message> msg) : _message(msg)
{
	_message->get_session()->set_status(session::NEW);
	cout << "handler::handler(): session " << _message->get_session()->get_id() << " started" << endl;
}

handler::~handler()
{
	cout << "handler::handler(): session " << _message->get_session()->get_id() << " stopped" << endl;
}


void handler::start()
{
	//cout << "handler::start(): this_thread=" << boost::this_thread::get_id() << endl;
	memset(_data, '\0', BUF_SIZE);
	mutable_buffers_1 buf = buffer(_data);
	_message->get_session()->get_socket()->async_receive(buf, boost::bind(&handler::read, shared_from_this(), placeholders::error, placeholders::bytes_transferred));
}


void handler::read(const boost::system::error_code& error, std::size_t bytes_transferred)
{
	if (!error)
	{
		//cout << "handler::read(): this_thread=" << boost::this_thread::get_id() << endl;
		if (_message->get_session()->get_status() != session::DONE)
		{
			shared_ptr<chat_line> line(new chat_line);
			line->parse(_data);
			_message->set_chat_line(line);
			//cout << "handler::read(): this_thread=" << boost::this_thread::get_id() << ", _message=" << _message->get_chat_line()->to_string() << endl;
			process();
			//cout << "handler::read(): this_thread=" << boost::this_thread::get_id() << ", _message=" << _message->get_chat_line()->to_string() << endl;
			_message->get_session()->get_socket()->async_send(buffer(_message->get_chat_line()->to_string()), boost::bind(&handler::send, shared_from_this(), placeholders::error, placeholders::bytes_transferred));
		}
		else
		{
			//cout << "handler::read(): this_thread=" << boost::this_thread::get_id() << ", destroying session" << endl;
		}
	}
}


void handler::send(const boost::system::error_code& error, std::size_t bytes_transferred)
{
	if (!error)
	{
		//cout << "handler::send(): this_thread=" << boost::this_thread::get_id() << ", bytes_transferred=" << bytes_transferred << endl;
		if (_message->get_session()->get_status() == session::DONE)
		{
			//cout << "handler::send(): session destroyed" << endl;
			return;
		}
		start();
	}
}


void handler::process()
{
	//cout << "handler::process(): processing session " << _message->get_session()->get_id() << endl;
	
	if (_message->get_session()->get_status() == session::DONE)
	{
		remove_user(_message->get_session()->get_id());
		//cout << "handler::process(): session done, removing user" << endl;
	}
	// check commands and their proper usage
	else if (_message->get_chat_line()->get_command() == "login")
	{
		if (_message->get_session()->get_status() == session::NEW)
		{
			string username = _message->get_chat_line()->get_username();
			string password = _message->get_chat_line()->get_password();
			if (handler::_members.count(username) > 0 && handler::_members[username] == password)
			{
				//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", login ok" << endl;
				_message->get_session()->set_status(session::ACTIVE);
				_mutex_users.lock();
				_users[_message->get_session()->get_id()] = username;
				//cout << "handler::process(): _users.size()=" << _users.size() << endl;
				_mutex_users.unlock();
				_message->get_chat_line()->set_command("login ok");
			}
			else
			{
				//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", login failed" << endl;
				_message->get_session()->set_status(session::DONE);
				remove_user(_message->get_session()->get_id());
				_message->get_chat_line()->set_command("login failed");
			}
		}
		else
		{
			//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", cannot login" << endl;
			_message->get_chat_line()->set_command("cannot login");
		}
	}
	else if (_message->get_chat_line()->get_command() == "logout")
	{
		//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", logout" << endl;
		_message->get_session()->set_status(session::DONE);
		remove_user(_message->get_session()->get_id());
		_message->get_chat_line()->set_command("logout ok");
	}
	else if (_message->get_chat_line()->get_command() == "send")
	{
		if (_message->get_session()->get_status() == session::ACTIVE)
		{
			if (_message->get_chat_line()->get_to() == "")
			{
				//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", send failed, no 'to' field" << endl;
				_message->get_chat_line()->set_command("send failed, no 'to' field");
			}
			else if (_message->get_chat_line()->get_chat_line() == "")
			{
				//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", send failed, no 'chat' field" << endl;
				_message->get_chat_line()->set_command("send failed, no 'chat' field");
			}
			else
			{
				_mutex_users.lock();
				string userFrom = _users[_message->get_session()->get_id()];
				_mutex_users.unlock();
				_mutex_inbox.lock();
				_inbox[_message->get_chat_line()->get_to()].push_back(From(userFrom, _message->get_chat_line()->get_chat_line()));
				_mutex_inbox.unlock();
				//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", send ok, from=" <<
				//	_message->get_chat_line()->get_from() << ", chat=" << _message->get_chat_line()->get_chat_line() << endl;
				_message->get_chat_line()->set_command("send ok");
			}
		}
		else
		{
			//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", send failed, not authorized" << endl;
			_message->get_session()->set_status(session::DONE);
			remove_user(_message->get_session()->get_id());
			_message->get_chat_line()->set_command("send failed, not authorized");
		}
	}
	else if (_message->get_chat_line()->get_command() == "receive")
	{
		if (_message->get_session()->get_status() == session::ACTIVE)
		{
			_mutex_users.lock();
			string username = _users[_message->get_session()->get_id()];
			_mutex_users.unlock();
			_mutex_inbox.lock();
			From fm;
			if (_inbox.count(username) > 0 && _inbox[username].size() > 0)
			{
				fm = _inbox[username].back();
				_inbox[username].pop_back();
			}
			_mutex_inbox.unlock();
			if (fm.empty())
			{
				_message->get_chat_line()->set_command("receive ok, no chatLine");
				//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", receive ok, no chatLine" << endl;
			}
			else
			{
				_message->get_chat_line()->set_command("receive ok");
				_message->get_chat_line()->set_chat_line(fm.chatLine);
				_message->get_chat_line()->set_from(fm.from);
				//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", receive ok, from=" <<
				//	fm.from << ", chat=" << fm.chatLine << endl;
			}
		}
		else
		{
			//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", receive failed, not authorized" << endl;
			_message->get_session()->set_status(session::DONE);
			remove_user(_message->get_session()->get_id());
			_message->get_chat_line()->set_command("receive failed, not authorized");
		}
	}
	else
	{
		//cout << "handler::process(): session=" << _message->get_session()->get_id() << ", unknown command" << endl;
		_message->get_chat_line()->set_command("unknown command");
	}
}


void handler::remove_user(long session_id)
{
	_mutex_users.lock();
	_users.erase(session_id);
	_mutex_users.unlock();
}
