
#ifndef HANDLER_HPP
#define HANDLER_HPP


#include <string>
#include <vector>
#include <boost/asio.hpp>
#include <boost/asio/ip/tcp.hpp>
#include <boost/enable_shared_from_this.hpp>
#include "message.hpp"


using std::string;
using std::vector;
using namespace boost::asio;
using boost::system::error_code;
using boost::system::system_error;
using boost::shared_ptr;


/**
Handles a client from the beginning until the end of session.
**/
class handler : public boost::enable_shared_from_this<handler>
{
public:

	/**
	Chat line with 'from' address.
	**/
	struct From
	{
		string from;
		
		string chatLine;
		
		From() : from(), chatLine()
		{
		}
		
		From(string frm, string line) : from(frm), chatLine(line)
		{
		}
		
		bool empty()
		{
			if (from == "" && chatLine == "")
				return true;
			return false;
		}
	};

	handler(shared_ptr<message> msg);

	~handler();
	
	void start();
	
	/**
	Username and password for each registered user.
	**/
	static map<string, string> _members;
	
private:

	void process();
	
	void remove_user(long session_id);
	
	shared_ptr<message> _message;
	
	static mutex _mutex_users;

	/**
	Maps session to username.
	**/
	static map<long, string> _users;

	static mutex _mutex_inbox;
	
	/**
	Inbox with messages for a specific user sent by an user.
	**/
	static map<string, vector<From> > _inbox;

	void read(const boost::system::error_code& error, std::size_t bytes_transferred);

	void send(const boost::system::error_code& error, std::size_t bytes_transferred);

	static const short BUF_SIZE = 128;

	char _data[BUF_SIZE];
};


typedef boost::shared_ptr<handler> handler_ptr;

#endif // HANDLER_HPP
