
#include "server.hpp"
#include <vector>
#include <boost/thread.hpp>
#include <boost/bind.hpp>


using std::cout;
using std::endl;
using std::vector;
using namespace boost::asio;
using boost::thread;
using boost::system::error_code;
using boost::system::system_error;
using boost::shared_ptr;


server::server(io_service& ios) : _ios(ios), _acceptor(_ios, ip::tcp::endpoint(ip::tcp::v4(), PORT))
{
	accept();
}


void server::run()
{
	const short THREAD_NO = 16;
	vector<shared_ptr<thread> > threads;
	for (int i = 0; i < THREAD_NO; ++i)
	{
		shared_ptr<thread> t(new thread(boost::bind(&io_service::run, &_ios)));
		threads.push_back(t);
	}
	
	for (int i = 0; i < THREAD_NO; ++i)
		threads[i]->join();
}


void server::accept()
{
	//cout << "server::accept(): this_thread=" << boost::this_thread::get_id() << endl;
	shared_ptr<ip::tcp::socket> client(new ip::tcp::socket(_ios));
	handler_ptr hnd(new handler(message::create(client)));
	_acceptor.async_accept(*client, boost::bind(&server::accepted, this, hnd, placeholders::error));
}


void server::accepted(handler_ptr hnd, const boost::system::error_code& error)
{
	if (!error)
	{
		hnd->start();
		//cout << "server::accepted(): this_thread=" << boost::this_thread::get_id() << endl;
	}
	accept();
}
