
#include "session.hpp"


using namespace boost::asio;
using boost::mutex;
using boost::shared_ptr;


mutex session::_mutex_counter;

long session::_session_counter = 0;


session::session(shared_ptr<ip::tcp::socket> socket) : _socket(socket), _status(NONE)
{
	_id = session::next_session_counter();
	time_t t;
	time(&t);
	_last_activity = t;
}


session::session(long id, time_t last_activity, shared_ptr<ip::tcp::socket> socket, status a_status) :
	_id(id), _last_activity(last_activity), _socket(socket), _status(a_status)
{
}


session::~session()
{
	_id = 0;
	_last_activity = 0;
}


long session::get_id()
{
	return _id;
}


shared_ptr<ip::tcp::socket> session::get_socket()
{
	return _socket;
}


void session::set_socket(shared_ptr<ip::tcp::socket> socket)
{
	_socket = socket;
}


time_t session::last_activity()
{
	return _last_activity;
}

	
void session::last_activity_now()
{
	time_t t;
	time(&t);
	_last_activity = t;
}


session::status session::get_status()
{
	return _status;
}

	
void session::set_status(session::status a_status)
{
	_status = a_status;
}


long session::next_session_counter()
{
	_mutex_counter.lock();
	long id = ++session::_session_counter;
	_mutex_counter.unlock();
	return id;
}
