
#ifndef SESSION_HPP
#define SESSION_HPP

#include <boost/asio.hpp>
#include <boost/asio/ip/tcp.hpp>
#include <boost/thread/mutex.hpp>


using namespace boost::asio;
using boost::system::error_code;
using boost::system::system_error;
using boost::mutex;
using boost::shared_ptr;


/**
Describes a client session (established via TCP).
It has unique ID, last activity time, corresponding socket.
**/
class session
{
public:

	enum status
	{
		NONE, // default
		NEW, // new conected client
		ACTIVE, // request/response generated
		DONE // processing finished on TCP and/or chat level
	};

	session(shared_ptr<ip::tcp::socket> socket);

	session(long session_id, time_t last_activity, shared_ptr<ip::tcp::socket> socket, status a_status);

	~session();

	long get_id();

	shared_ptr<ip::tcp::socket> get_socket();

	void set_socket(shared_ptr<ip::tcp::socket> socket);
	
	time_t last_activity();
	
	void last_activity_now();
	
	status get_status();
	
	void set_status(status a_status);
	
private:

	static long next_session_counter();

	static mutex _mutex_counter;

	static long _session_counter;

	long _id;

	time_t _last_activity;

	shared_ptr<ip::tcp::socket> _socket;

	status _status;
};


#endif // SESSION_HPP
